//= require <gettexthelper>
//= require <messagebus>
//= require <statusbar>
//= require <navpanel>
//= require <generated/config>

Ext.namespace('dSS11', 'dSS11.config', 'dSS', 'dSA');

function GoogleMapsCallback() {
    dSS11.MessageBus.fireEvent('googlemapsloaded');
};

dSS11.CreateMainViewport = function(panel)
{
    /* for some reason this check is also triggered on IE9, need a fix in ExtJS
    if (Ext.isIE6 === true) {
        Ext.Msg.alert(_("Error"),
                      _("Internet Explorer 6 is not supported, please " +
                        "upgrade your webbrowser!"));
        return;
    }
    */

    var viewport = new Ext.Viewport(
    {
        hideBorders: true,
        layout: 'border',
        items:
            [
                {
                    xtype: 'box',
                    region: 'north',
                    margins: { 'top': 10, left: 10, right: 10 },
                    height: 52,
                    autoEl:
                    {
                        tag: 'div',
                        html: '<a href="http://digitalstrom.org/" alt="' +
                                _("visit digitalSTROM.org") +
                                '" target="_blank"><img id="logo"' +
                                ' src="images/ds_logo.png"' +
                                ' alt="digitalSTROM Logo"></a>' +
                                '<h1>' + _("digitalSTROM Configurator") +
                                '</h1>'
                    }
                },
                panel
            ]
    });

    if (dSS11.config.dSAIntegration === true) {
        dSA.config.language = dSS11.config.language;
        dSA.LoadLangpack(null,
            dSS11.StatusBar.setMessage.createDelegate(dSS11.StatusBar));
    }

    // a system update is currently in progress, mask out the UI, navpanel will
    // automatically enable advanced view, go to the update menu and launch
    // the "please wait" animation
    if (dSS11.config.updateInProgress === true) {
        viewport.el.mask();
    }
    var params = Ext.urlDecode(window.location.search.substring(1));
    if ((dSS11.config.updateInProgress === true) ||
        (params.showupdate == 1)) {
        dSS11.StatusBar.showAdvancedView();
        viewport.dSS11Main.setActiveTab(viewport.dSS11Main.system);
    }
    return viewport;
};

dSS11.createMainPanel = function(enableModule) {
    var listeners;

    var items =
    [
            dSS11.AddOnsPanel
    ];

    if (dSS11.config.dSSIntegration === true) {
        items = items.concat(
        [
            {
                xtype: 'dssscenespanel',
                title: _("Scenes"),
                ref: 'scenes'
            },
            {
                xtype: 'dsszonebrowser',
                title: _("Zones"),
                ref: 'zones'
            }
        ]);

    }

    items.push(
        {
            xtype: 'dss11navpanel',
            title: _("System"),
            ref: 'system'
        }
    );


    if (dSS11.config.dSSIntegration === true) {
        items.push(
            {
                title: dSS._("Hardware"),
                xtype: 'dsmoverviewpanel',
                ref: 'circuits'
            }
        );
    }

    items.push(
        {
            title: _("Help"),
            ref: 'help',
            xtype: 'dss11help'
        }
    );

    var panel =
    {
        xtype: 'tabpanel',
        region: 'center',
        ref: 'dSS11Main',
        activeTab: 0,
        items: items,
        bbar: dSS11.StatusBar,
        listeners: {
            afterrender: function() {
                if (dSS11.config.dSSIntegration === true) {
                    this.hideTabStripItem(this.circuits);
                }
                this.hideTabStripItem(this.system);

                if (dSS11.config.dSSIntegration === true) {
                    dSS11.GetImageVersion();
                }
            },
            refreshview: function() {
                var active = this.getActiveTab();
                if (dSS11.config.dSSIntegration === true) {
                    if ((active === this.circuits) ||
                        (active === this.scenes) ||
                        (active === this.zones)) {
                        dSS.MessageBus.fireEvent('reloadapartment');
                        dSS.config.firstLoad = false;
                    } else {
                        dSS.MessageBus.fireEvent('refreshview');
                    }
                }

                if (active === this.addons) {
                    dSS11.MessageBus.fireEvent('refreshaddonsview');
                } else if (active === this.system) {
                    dSS11.MessageBus.fireEvent('refreshsystemupdateview');
                }
            },
            switchview: function(advanced) {
                if (advanced === true) { // go advanced
                    if (dSS11.config.dSSIntegration === true) {
                        this.unhideTabStripItem(this.circuits);
                    }
                    this.unhideTabStripItem(this.system);
                } else { // go default
                    var active = this.getActiveTab();

                    if (dSS11.config.dSSIntegration === true) {
                        if ((active === this.system) ||
                            (active === this.circuits)) {
                            this.setActiveTab(this.scenes);
                        }

                        this.hideTabStripItem(this.circuits);
                    } else {
                        if (active === this.system) {
                            this.setActiveTab(0);
                        }
                    }

                    this.hideTabStripItem(this.system);
                }
            } // switchview
        } // listeners
    };


    return panel;
};

dSS11.GetImageVersion = function() {
    Ext.Ajax.request({
        url: "/cgi-bin/imgversion.sh",
        disableCaching: true,
        success: function(response, options) {
            try {
                var json = Ext.decode(response.responseText);
                if (json.success === true) {
                     dSS11.StatusBar.setVersion(json.data.version);
                     return;
                }

                dSS11.StatusBar.setMessage(
                    _("Could not load version information") + '.');
            } catch(error) {
                dSS11.StatusBar.setMessage(
                    _("Could not load version information") + ': ' + error);
            }
        },
        failure: function(response, options) {
            dSS11.StatusBar.setMessage(_("Could not load version information") +
                                       '.');
        }
    });
};

dSS11.dSSCheck = function()
{
    // initialize dSS gettext
    Ext.Ajax.request({
        url: '/json/system/version',
        scope: this,
        method: "GET",
        disableCaching: true,
        success: function(response, options) {
            try {
                if (response.responseText.length === 0) {
                    dSS11.config.dSSIntegration = false;
                    Ext.Msg.alert(_("Error"),
                        _("No response from dSS, proceeding in limited mode." +
                          " Check if the dSS is running and reload this page " +
                          "again!"));
                    dSS11.StatusBar.setMessage(_("No response from dSS, please check if dSS is running"));
                    this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
                    return;

                }

                this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
            } catch (error) {
                dSS11.config.dSSIntegration = false;
                Ext.Msg.alert(_("Error"), error.message);
                this.viewport = dSS11.CreateMainViewport(
                                                dSS11.createMainPanel());
            }
        },
        failure: function(response, options) {
            dSS11.config.dSSIntegration = false;
            Ext.Msg.alert(_("Error"),
                _("No response from dSS, proceeding in limited mode." +
                  " Check if the dSS is running and reload this page " +
                  "again!"));
                dSS11.StatusBar.setMessage(_("No response from dSS, please check if dSS is running"));

            this.viewport = dSS11.CreateMainViewport(dSS11.createMainPanel());
        }
    });
};

dSS11.loadDSSLangpack = function()
{
    Ext.Ajax.request({
        url: dSS.config.langPath,
        scope: this,
        method: "GET",
        disableCaching: true,
        success: function(response, options) {
            try {
                var data = Ext.util.JSON.decode(response.responseText);

                var params = {
                                'domain' : 'dss-installer',
                                'locale_data' : { 'dss-installer' : data }
                             };
                dSS.gt = new Gettext(params);
                dSS11.dSSCheck();
            } catch(err) {
                Ext.Msg.alert(_("Error"),
                              _("Could not load language package") + ': ' +
                              err);
                dSS11.dSSCheck();
            }
        },
        failure: function(response, options) {
            Ext.Msg.alert(_("Error"),
                          _("Could not load language packge"));
            dSS11.dSSCheck();
        }
    });
};

dSS11.localizeExt = function()
{
    Ext.MessageBox.buttonText.yes = _("Yes");
    Ext.MessageBox.buttonText.no = _("No");
    Ext.MessageBox.buttonText.cancel = _("Cancel");
};

dSS11.LoadLangpack = function()
{
    Ext.Ajax.request({
        url            : dSS11.config.langPath + 'langpack.json',
        disableCaching : true,
        scope          : this,
        success        : function(response, options) {
            try{
                var data = Ext.decode(response.responseText);
                var params = {
                                "domain" : "dss11",
                                "locale_data" : { "dss11" : data }
                             };

                dSS11.config.gt = new Gettext(params);

                dSS11.localizeExt();

                if (dSS11.config.dSSIntegration === true)  {
                    dSS11.loadDSSLangpack();
                } else {
                    this.viewport = dSS11.CreateMainViewport(
                                            dSS11.createMainPanel());
                }
            } catch(error) {
                Ext.Msg.alert(_("Error"),
                        _("Could not load language package") + ': ' +
                        error.message);

                if (dSS11.config.dSSIntegration === true)  {
                    dSS11.loadDSSLangpack();
                } else {
                    this.viewport = dSS11.CreateMainViewport(
                                            dSS11.createMainPanel());
                }
            }
        },
        failure : function(response, options) {
            Ext.Msg.alert(_("Error"), _("Could not load language package"));
            if (dSS11.config.dSSIntegration === true)  {
                dSS11.loadDSSLangpack();
            } else {
                this.viewport = dSS11.CreateMainViewport(
                                        dSS11.createMainPanel());
            }
        }
    });
};

Ext.onReady(
    function()
    {
        Ext.get('start').remove();
        dSS11.config.dSS11TimeDiff = 0;
        dSS11.config.timeUpdateTimeout = 0;
        if (dSS.module === true) {
            dSS11.config.dSSIntegration = true;
        } else {
            dSS11.config.dSSIntegration = false;
        }

        if (dSA.module === true) {
            dSS11.config.dSAIntegration = true;
        } else {
            dSS11.config.dSAIntegration = false;
        }

        dSS11.config.language = 'de_DE';
        dSS11.config.langPath = '/locale/' + dSS11.config.language +
                                '/LC_MESSAGES/';

        dSS11.config.rebootWait = 70; // seconds to wait for reboot
        dSS11.config.codes = {
            "ERR_OK"        : 0,
            "ERR_MODE"      : 1,
            "ERR_MASK"      : 2,
            "ERR_IP"        : 3,
            "ERR_GW"        : 4,
            "ERR_DNS"       : 5,
            "ERR_FWRITE"    : 6,
            "ERR_NETDOWN"   : 7,
            "ERR_NETUP"     : 8,
            "ERR_PARAM"     : 9,
            "ERR_REDIRECT"  : 10,
            "ERR_RESTORE"   : 11,
            "ERR_BACKUP"    : 12,
            "ERR_HOSTNAME"  : 13,
            "ERR_APPSTART"  : 14,
            "ERR_REBOOT"    : 15,
            "ERR_HALT"      : 16,
            "ERR_NOZONES"   : 17,
            "ERR_NONTP"     : 18,
            "ERR_NONTPSRV"  : 19,
            "ERR_LINK"      : 20,
            "ERR_DATE"      : 21,
            "ERR_TIME"      : 22,
            "ERR_PASSWORD"  : 23,
            "ERR_SETUP"     : 24,
            "ERR_DOWAIT"    : 25,
            "ERR_UPDATE"    : 26,
            "ERR_UPNPNAME"  : 27,
            "ERR_DSMFW"     : 28,
            "ERR_DISCLAIMER": 29,
            "ERR_FILENOTFOUND" : 30,
            "ERR_XML"       : 31,
            "ERR_INVALID"   : 32,
            "ERR_QUIT"      : 33,
            "ERR_REDIRECT_NAME": 34,
            "ERR_OPKG"      : 35,
            "ERR_USAGE"     : 64,   /* command line usage error **/
            "ERR_DATAERR"   : 65,	/* data format error **/
            "ERR_NOINPUT"   : 66,	/* cannot open input **/
            "ERR_NOUSER"    : 67,	/* addressee unknown **/
            "ERR_NOHOST"    : 68,	/* host name unknown **/
            "ERR_UNAVAILABLE" : 69,	/* service unavailable **/
            "ERR_SOFTWARE"  : 70,	/* internal software error **/
            "ERR_OSERR"     : 71,	/* system error (e.g., can't fork) **/
            "ERR_OSFILE"    : 72,	/* critical OS file missing **/
            "ERR_CANTCREAT" : 73,	/* can't create (user) output file **/
            "ERR_IOERR"     : 74,	/* input/output error **/
            "ERR_TEMPFAIL"  : 75,	/* temp failure; user is invited to retry */
            "ERR_PROTOCOL"  : 76,	/* remote error in protocol **/
            "ERR_NOPERM"    : 77,	/* permission denied **/
            "ERR_CONFIG"    : 78	/* configuration error **/
        };

        dSS11.config.opkgcodes = {
            "ERR_UNSPECIFIED"               : 0x00000001,
            "ERR_DOWNLOAD_FAILED"           : 0x00000002,
            "ERR_PACKAGE_ALREADY_INSTALLED" : 0x00000008,
            "ERR_PACKAGE_NOT_DOWNGRADED"    : 0x00001000
        };

        dSS11.config.err2text = function (errcode) {
			if (errcode === dSS11.config.codes.ERR_OK) return _("OK");
            if (errcode === dSS11.config.codes.ERR_USAGE) return _("command line usage error");
            if (errcode === dSS11.config.codes.ERR_DATAERR) return _("data format error");
            if (errcode === dSS11.config.codes.ERR_NOINPUT) return _("cannot open input");
            if (errcode === dSS11.config.codes.ERR_NOUSER) return _("addressee unknown");
            if (errcode === dSS11.config.codes.ERR_NOHOST) return _("host name unknown");
            if (errcode === dSS11.config.codes.ERR_UNAVAILABLE) return _("service unavailable");
            if (errcode === dSS11.config.codes.ERR_SOFTWARE) return _("internal software error");
            if (errcode === dSS11.config.codes.ERR_OSERR) return _("system error");
            if (errcode === dSS11.config.codes.ERR_OSFILE) return _("critical file missing");
            if (errcode === dSS11.config.codes.ERR_CANTCREAT) return _("can't create output file");
            if (errcode === dSS11.config.codes.ERR_IOERR) return _("input/output error");
            if (errcode === dSS11.config.codes.ERR_TEMPFAIL) return _("temporary failure, try again later");
            if (errcode === dSS11.config.codes.ERR_PROTOCOL) return _("remote error in protocol");
            if (errcode === dSS11.config.codes.ERR_NOPERM) return _("permission denied");
            if (errcode === dSS11.config.codes.ERR_CONFIG) return _("configuration error");
            return _("unkown error code") + ": " + errorcode;
        };

        dSS11.config.opkgstate = {
            "OPKG_NOT_INSTALLED"    : 1,
            "OPKG_UNPACKED"         : 2,
            "OPKG_HALF_CONFIGURED"  : 3,
            "OPKG_INSTALLED"        : 4,
            "OPKG_HALF_INSTALLED"   : 5,
            "OPKG_CONFIG_FILES"     : 6,
            "OPKG_POST_INST_FAILED" : 7,
            "OPKG_REMOVAL_FAILED"   : 8,
            "OPKG_LAST_STATE_STATUS": 9
        };

        dSS11.config.apartmentid = {
            "MIN" : 0,
            "MAX" : 62
        };

        // automatically reload UI after update of following packages
        dSS11.config.reloadUIAfterUpdate =
        [
            'dss11-websetup', 'dss-web-module', 'dsa-ui-module', 'dss',
            'task-system-addons'
        ];

        Ext.QuickTips.init();
        Ext.apply(Ext.QuickTips.getQuickTip(), { maxWidth: 500 });

        dSS11.config.updateInProgress = false;

        Ext.Ajax.request({
            url             : '/cgi-bin/updatestatuswrapper.sh',
            disableCaching  : true,
            method          : 'GET',
            scope           : this,
            params          : { type : 'active' },
            success         : function(response, options) {
                try {
                    var data = Ext.decode(response.responseText);
                    if ((data.success === true) &&
                        (data.code === dSS11.config.codes.ERR_DOWAIT)) {
                            // load the bare minimum, UI will be reloaded after
                            // the update anyway
                            dSS11.config.updateInProgress = true;
                            dSS11.config.dSSIntegration = false;
                            dSS11.config.dSAIntegration = false;
                        }
                    dSS11.LoadLangpack();
                } catch(error) {
                    dSS11.LoadLangpack();
                }
            },
            failure : function(response, options) {
                dSS11.LoadLangpack();
            }
        });
    }
);

